import gtk
import gtk.glade

from Source import AptSource, DebSource

class DialogAptSourceEdit:
    def __init__(self, parent, source_entry, datadir):
        self.source_entry = source_entry
        self.gladexml = gtk.glade.XML(datadir+'/glade/dialogs.glade')
        self.dialog = self.gladexml.get_widget('dialog_aptedit')
        self.dialog.set_transient_for(parent)
        self.button_edit = self.gladexml.get_widget('button_aptedit')
        
        self.entry_name = self.gladexml.get_widget('entry_aptedit_name')
        self.entry_name.set_text(source_entry.get_name())
        self.entry_name.connect('changed', self.check_entry)
        
        self.entry_pkg = self.gladexml.get_widget('entry_aptedit_pkg')
        self.entry_pkg.set_text(source_entry.get_package())
        self.entry_pkg.connect('changed', self.check_entry)
        
        self.entry_key = self.gladexml.get_widget('entry_aptedit_key')
        key = source_entry.get_key()
        if key: self.entry_key.set_text(key)
        
        self.entry_bin = self.gladexml.get_widget('entry_aptedit_bin')
        bin = source_entry.get_bin_repo()
        if bin: self.entry_bin.set_text(bin)
        
        self.entry_src = self.gladexml.get_widget('entry_aptedit_src')
        src = source_entry.get_src_repo()
        if src: self.entry_src.set_text(src)
        
    def run(self):
        res = self.dialog.run()
        self.dialog.hide()
        if res == gtk.RESPONSE_OK:
            name = self.entry_name.get_text()
            pkg = self.entry_pkg.get_text()
            key = self.entry_key.get_text()
            bin = self.entry_bin.get_text()
            src = self.entry_src.get_text()
            if key == "": key = None
            if bin == "": bin = None
            if src == "": src = None
            source = AptSource(name, pkg, key, bin, src)
            return source
        
    def check_entry(self, *args):
        if self.entry_name.get_text() == "" or self.entry_pkg.get_text() == "":
            self.button_edit.set_sensitive(False)
        else:
            self.button_edit.set_sensitive(True)
        
class DialogDebSourceEdit:
    def __init__(self, parent, source_entry, datadir):
        self.source_entry = source_entry
        self.gladexml = gtk.glade.XML(datadir+'/glade/dialogs.glade')
        self.dialog = self.gladexml.get_widget('dialog_debedit')
        self.dialog.set_transient_for(parent)
        self.button_edit = self.gladexml.get_widget('button_debedit')
        
        self.entry_name = self.gladexml.get_widget('entry_debedit_name')
        self.entry_name.set_text(source_entry.get_name())
        self.entry_name.connect('changed', self.check_entry)
        
        self.entry_pkg = self.gladexml.get_widget('entry_debedit_pkg')
        self.entry_pkg.set_text(source_entry.get_package())
        self.entry_pkg.connect('changed', self.check_entry)
        
        self.entry_ver = self.gladexml.get_widget('entry_debedit_ver')
        ver = source_entry.get_version()
        if ver: self.entry_ver.set_text(ver)
        
        self.entry_url = self.gladexml.get_widget('entry_debedit_url')
        self.entry_url.set_text(source_entry.get_url())
        
    def run(self):
        res = self.dialog.run()
        self.dialog.hide()
        if res == gtk.RESPONSE_OK:
            name = self.entry_name.get_text()
            pkg = self.entry_pkg.get_text()
            ver = self.entry_ver.get_text()
            url = self.entry_url.get_text()
            if ver == "": ver = None
            source = DebSource(name, pkg, ver, url)
            return source
    
    def check_entry(self, *args):
        if self.entry_name.get_text() == "" or self.entry_pkg.get_text() == "" or self.entry_url.get_text() == "":
            self.button_edit.set_sensitive(False)
        else:
            self.button_edit.set_sensitive(True)