#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vowel - VowelMain.py
# VOWEL is Easy Application Installer On Ubuntu
# This is meant to install applications unsupported in ubuntu repository
# License:: GPL
# Author:: Chang-ho Woo(mailto:zodiac12k@gmail.com)
# Blog:: http://zodiac12k.egloos.com

import os
import sys
import time

import Check
from AptWorker import AptWorker
from DebWorker import DebWorker
from DBConnection import DBConnection
from Source import AptSource, DebSource

# Supported Architecture
DISTRO = 'ubuntu'
CODE = 'gutsy hardy'
MACHINE = 'x86 x86_64'
# Dependencies
DEPEDENCY = ['dpkg', 'grep', 'lsb-release', 'python-pysqlite2']

class MenuTree:
    def page(self):
        left_margin = 7
        border = ' ' * 4
        lines = []
        lenlinetxt = []
        for i in self.linetxt:#calculate the lenght of each element of linetxt and append them to lenlinetx
            lenlinetxt.append(len(i))
        
        diffs = []
        diffspc = []
        it = 0#iterator
        it <= len(self.linetxt)#make sure it's not out of range
        for line in self.linetxt:
            maxlen = max(lenlinetxt)
            diffstmp = (maxlen - lenlinetxt[it]) / 2
            diffs.append(diffstmp)
            if it == 0 or it == len(self.linetxt)-1:#treat the 1st and the last line differently
                diffspctmp = '-' * diffs[it]#put -
                rightmargin = '-' * (maxlen - lenlinetxt[it] )
            else:
                diffspctmp = ' ' * diffs[it]#put ' '
                rightmargin = ' ' * (maxlen - lenlinetxt[it] )
            diffspc.append(diffspctmp)
            genline = self.linetxt[it] + rightmargin #diffspc[it] +
            lines.append(genline)
            it += 1
        
        maxintlen = len(border)*2 + max(diffs)*2
        print '\n'
        print ' ' * left_margin + '+' + ('-' * len(border)) + lines[0] + ('-' * len(border)) + '+'#1st line
        for line in lines[1:-1]:
            print ' ' * left_margin + '|' + border + line + border + '|'
        print ' ' * left_margin + '+' + ('-' * len(border)) + lines[len(self.linetxt)-1] + ('-' * len(border)) + '+'#last line
        linetxt2 = []
        for l in lines:#calculate the lenght of each element of linetxt and append them to lenlinetxt
            linetxt2.append(len(l))
            
class VowelMain:
    def __init__(self, dbpath):
        self.dbpath = dbpath
        self.con = DBConnection()
        self.code = self.codename()
        self.machine = self.machinename()
        
        self.aptsources = []
        self.debsources = []
        
    def check_environments(self):
        """ Check environments(network, system requirements, and so on)
            under Terminal User Interface """
        print '\n네트워크 연결 테스트 중...'
        if self.net_test() == False:
            print '\nERROR: 인터넷에 연결되어 있지 않습니다. \033[1m종료...\033[0m\n'
            sys.exit(1)
        print '\n프로그램 요구 사항 점검 중...'
        name = self.distroname()
        if self.distro_check(name) == False:
            print 'ERROR: 이 프로그램은 ' + name + '을/를 지원하지 않습니다. \033[1m종료...\033[0m\n'
            sys.exit(1)
        name = self.codename()

    def distroname(self):
        """ Get your system's distributor id """
        return Check.Architecture().distroname()
    
    def codename(self):
        """ Get your system's codename """
        return Check.Architecture().codename()

    def machinename(self):
        """ Get your hardware machine name """
        return Check.Architecture().machinename()
      
    def net_test(self):
        """ Test your network """
        net = Check.Network()
        return net.test()   

    def distro_check(self, name):
        """ Check if your distributor id meets this program's requirements """
        arch = Check.Architecture()
        return arch.check_distro(name, DISTRO)
    
    def code_check(self, name):
        """ Check if your codename meets this program's requirements """
        arch = Check.Architecture()
        return arch.check_code(name, CODE)
    
    def machine_check(self, name):
        """ Check if your hardware machine name meets this program's requirements """
        arch = Check.Architecture()
        return arch.check_machine(name, MACHINE)
    
    def dep_check(self):
        """ Check dependencies """
        dep = Check.Dependency()
        dep.checkntry(dependencies)

    def apt_install(self, aptsource):
        """ Install an application from an apt source
            under Terminal User Interface """
        apt = Apt(aptsource)
        apt.write_sourceslist()
        apt.install()
        
    def deb_install(self, debsource):
        """ Install an application from an deb source
            under Terminal User Interface """
        package = debsource.get_package().encode('utf-8')
        version = debsource.get_version()
        pkg = Check.Package(package)
        
        if pkg.isinstalled() == True:
            if pkg.islatest(version) == True:
                query = ' 꾸러미는 이미 최신 버전입니다. 재설치하시겠습니까[y or n]?'
            else:
                query = ' 꾸러미는 최신 버전이 아닙니다. 최신 버전으로 업그레이드하시겠습니까[y or n]?'
                
            while True:
                reply = raw_input('Vowel: ' + package + query)
                if reply.lower() in ('y', 'yes'): break
                elif reply.lower() in ('n', 'no'): return
                else: continue
            
        deb = Deb(debsource)
        deb.download()
        deb.install()
        
        while True:
            reply = raw_input('Vowel: 다운받은 꾸러미를 디스크에서 삭제할까요[y or n]?')
            if reply.lower() in ('y', 'yes'):
                deb.remove()
                break
            elif reply.lower() in ('n', 'no'): break
            else: continue
            
    def get_aptsources(self):
        """ Get apt source entries from database """
        del self.aptsources[0:]
        if self.con.connect(self.dbpath) == False:
            return False
        self.con.cursor()
        table = 'aptsource'
        statement = "select * from %s where code=? order by name" % table
        if self.con.execute(statement, (self.code,)) == False:
            return False
        data = self.con.fetchall()
        for name, package, key, bin_repo, src_repo, code in data:
            self.aptsources.append(AptSource(name, package, key, bin_repo, src_repo))
        return True
    
    def get_debsources(self):
        """ Get deb source entries from database """
        del self.debsources[0:]
        if self.con.connect(self.dbpath) == False:
            return False
        self.con.cursor()
        table = 'debsource'
        statement = "select * from %s where code=? and machine=? order by name" % table
        if self.con.execute(statement, (self.code, self.machine)) == False:
            return False
        data = self.con.fetchall()
        for name, package, version, url, code, machine in data:
            self.debsources.append(DebSource(name, package, version, url))
        return True
    
    def insert_aptsource(self, source):
        """ Insert Query for aptsource table
            Insert into aptsource table a source entry """
        if self.con.connect(self.dbpath) == False:
            return False
        self.con.cursor()
        table = 'aptsource'
        statement = "insert into %s (name, package, key, bin_repo, src_repo, code) values (?, ?, ?, ?, ?, ?)" % table
        if self.con.execute(statement, (source.get_name(),
                                        source.get_package(),
                                        source.get_key(),
                                        source.get_bin_repo(),
                                        source.get_src_repo(),
                                        self.code)) == False:
            return False
        self.con.commit()
        return True
    
    def update_aptsource(self, source, name):
        """ Update Query for aptsource table
            Update aptsource table set a source entry """
        if self.con.connect(self.dbpath) == False:
            return False
        self.con.cursor()
        table = 'aptsource'
        statement = "update %s set name=?, package=?, key=?, bin_repo=?, src_repo=? where code=? and name=?" % table
        if self.con.execute(statement, (source.get_name(),
                                        source.get_package(),
                                        source.get_key(),
                                        source.get_bin_repo(),
                                        source.get_src_repo(),
                                        self.code,
                                        name)) == False:
            return False
        self.con.commit()
        return True
        
    def delete_aptsource(self, source):
        """ Delete Query for aptsource table
            Delete from aptsource table where """
        if self.con.connect(self.dbpath) == False:
            return False
        self.con.cursor()
        table = 'aptsource'
        statement = "delete from %s where code=? and name=?" % table
        if self.con.execute(statement, (self.code, source.get_name())) == False:
            return False
        self.con.commit()
        return True
    
    def insert_debsource(self, source):
        """ Insert Query for debsource table
            Insert into aptsource table a source entry """
        if self.con.connect(self.dbpath) == False:
            return False
        self.con.cursor()
        table = 'debsource'
        statement = "insert into %s (name, package, version, url, code, machine) values (?, ?, ?, ?, ?, ?)" % table
        if self.con.execute(statement, (source.get_name(),
                                        source.get_package(),
                                        source.get_version(),
                                        source.get_url(),
                                        self.code,
                                        self.machine)) == False:
            return False
        self.con.commit()
        return True

    def update_debsource(self, source, name):
        """ Update Query for debsource table
            Update aptsource table set a source entry """
        if self.con.connect(self.dbpath) == False:
            return False
        self.con.cursor()
        table = 'debsource'
        statement = "update %s set name=?, package=?, version=?, url=? where code=? and machine=? and name=?" % table
        if self.con.execute(statement, (source.get_name(),
                                        source.get_package(),
                                        source.get_key(),
                                        source.get_bin_repo(),
                                        source.get_src_repo(),
                                        self.code,
                                        self.machine,
                                        name)) == False:
            return False
        self.con.commit()
        return True
        
    def delete_debsource(self, source):
        """ Delete Query for aptsource table
            Delete from aptsource table where """
        if self.con.connect(self.dbpath) == False:
            return False
        self.con.cursor()
        table = 'debsource'
        statement = "delete from %s where code=? and machine=? and name=?" % table
        if self.con.execute(statement, (self.code, self.machine, source.get_name())) == False:
            return False
        self.con.commit()
        return True

    def show_menu(self, name):
        """ Show the main menu
            under Terminal User Interface """
        menu = MenuTree()
        menu.linetxt = ['', 'VOWEL Menu ver.' + ver]
        for i in name:
            menu.linetxt.extend(['', i])
            
        menu.linetxt.extend(['', '0 - Exit', '', 'NOTE: IF THE SCREEN TURNS BLACK, PLEASE TYPE ALT+F1', ''])
        menu.page()
        
    def run(self):
        """ Run the terminal program """
        self.get_aptsources()
        self.get_debsources()
        print self.sourcelist
        src_length = len(self.sourcelist)    
        menuline = []
        for i in range(src_length):
            menuline.append(str(i+1) + ' - ' + self.sourcelist[i].get_name())
        
        while True:
            self.show_menu(menuline)
            
            try:
                choice = int(raw_input('\nVowel: 설치하고 싶은 프로그램의 번호를 입력하고 ENTER키를 누르세요. '))
            except ValueError:
                continue
            else:
                if choice == 0: break
                elif choice > src_length: continue
                else:
                    if isinstance(self.sourcelist[choice-1], AptSource):
                        self.apt_install(self.sourcelist[choice-1])
                    elif isinstance(self.sourcelist[choice-1], DebSource):
                        self.deb_install(self.sourcelist[choice-1])