#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vowel - VowelMainGtk.py
# VOWEL is Easy Application Installer On Ubuntu
# This is meant to install applications unsupported in ubuntu repository
# License:: GPL
# Author:: Chang Ho Woo(mailto:zodiac12k@gmail.com)
# Blog:: http://zodiac12k.egloos.com

import pygtk
pygtk.require('2.0')
import gtk
import gtk.gdk
import gtk.glade

import os
import sys
import time
import gobject
import gettext
from gettext import gettext as _

import SimpleGladeApp
from SimpleGladeApp import SimpleGladeApp

from VowelMain import VowelMain
from AptWorker import AptWorker
from DebWorker import DebWorker
from DialogAdd import *
from DialogEdit import *

VERSION = '0.3.1'

(COLUMN_TOGGLE, COLUMN_DESC) = range(2)

(
    STORE_TOGGLE,
    STORE_NAME,
    STORE_SOURCE
) = range(3)

(
    STORE_DEBNAME,
    STORE_DEBSOURCE
) = range(2)

class VowelMainGtk(SimpleGladeApp, VowelMain):
    def __init__(self, datadir=None):
        gtk.window_set_default_icon_name('vowel')
        VowelMain.__init__(self, datadir+'/data/source.sqlite')
        SimpleGladeApp.__init__(self, datadir+'/glade/main.glade', domain='vowel')
        
        self.check_environments()
        self.datadir = datadir
        self.window_main.realize()
        self.window_main.grab_focus()
        self.button_close.grab_focus()
        
        self.install_dict = {}
        self.install_count = 0
        
        self.init_aptstore()
        self.init_debstore()
        
        self.fill_aptstore()
        self.fill_debstore()
        
        self.window_main.show()
        
    def check_environments(self):
        self.window_start.realize()
        self.window_start.show()
        
        # Test network connectivity
        self.label_start.set_text("네트워크 연결 테스트 중...")
        self.update()
        time.sleep(0.5)
        net = self.net_test()
        self.check_error(net, header="네트워크 연결 오류", message="인터넷이 연결되어 있지 않습니다.")
        self.progressbar_start.set_fraction(0.3)
        self.update()
        
        # Graphical software requirements check process
        self.label_start.set_text("프로그램 요구 사항 점검 중...")
        self.update()
        time.sleep(0.1)
        name = self.distroname()
        ch = self.distro_check(name)
        msg = _("이 프로그램은 "+name+" 을(를) 지원하지 않습니다."
                " 프로그램 요구 사항을 확인하세요")
        self.check_error(ch, header="시스템 환경 오류", message=msg)
        self.progressbar_start.set_fraction(0.4)
        self.update()

        name = self.codename()
        ch = self.code_check(name)
        msg = _("이 프로그램은 "+name+" 을(를) 지원하지 않습니다."
                " 프로그램 요구 사항을 확인하세요.")
        self.check_error(ch, header="시스템 환경 오류", message=msg)
        self.progressbar_start.set_fraction(0.5)
        self.update()
        
        name = self.machinename()
        ch = self.machine_check(name)
        msg = _("이 프로그램은 "+name+" 을(를) 지원하지 않습니다."
                " 프로그램 요구 사항을 확인하세요.")
        self.check_error(ch, header="시스템 환경 오류", message=msg)
        self.progressbar_start.set_fraction(0.6)
        self.update()
        
        # Graphical dependency check process
        self.label_start.set_text("의존성 확인 중...")
        self.update()
        time.sleep(0.1)
        self.progressbar_start.set_fraction(0.8)
        self.update()
        
        # Get data sources which are used on this program
        self.label_start.set_text("데이터를 가져오는 중...")
        self.update()
        time.sleep(0.1)
        ch = self.get_aptsources() and self.get_debsources()
        msg = _("데이터베이스 파일을 열 수 없습니다."
                " 데이터베이스 파일이 존재하는지 확인하세요.")
        self.check_error(ch, header="데이터베이스 오류", message=msg)
        self.progressbar_start.set_fraction(1.0)
        self.update()
        
        self.window_start.destroy()
    
    def check_error(self, check, title="", header="", message=""):
        try:
            if check == False:
                raise Exception
        except Exception, e:
            self.window_start.destroy()
            d = gtk.MessageDialog(flags=gtk.DIALOG_MODAL,
                                  type=gtk.MESSAGE_ERROR,
                                  buttons=gtk.BUTTONS_CLOSE)
            d.set_title(title)
            d.set_markup("<big><b>%s</b></big>\n\n%s" % (header, message))
            d.realize()
            d.window.set_functions(gtk.gdk.FUNC_MOVE)
            d.run()
            d.destroy()
            sys.exit(1)
    
    def init_aptstore(self):
        self.aptsource_store = gtk.ListStore(gobject.TYPE_BOOLEAN,
                                             gobject.TYPE_STRING,
                                             gobject.TYPE_PYOBJECT)
        self.treeview_aptinstall.set_model(self.aptsource_store)
        
        cell_desc = gtk.CellRendererText()
        cell_desc.set_property('xpad', 6)
        cell_desc.set_property('ypad', 6)
        col_desc = gtk.TreeViewColumn('Application', cell_desc, markup=COLUMN_DESC)
        #col_desc.set_max_width(1000)
        
        cell_toggle = gtk.CellRendererToggle()
        cell_toggle.set_property('xpad', 6)
        cell_toggle.set_property('ypad', 6)
        cell_toggle.connect('toggled', self.on_toggled)
        col_toggle = gtk.TreeViewColumn('Toggle', cell_toggle, active=COLUMN_TOGGLE)
        
        self.treeview_aptinstall.append_column(col_toggle)
        #self.treeview_aptinstall.set_search_column(COLUMN_DESC)
        self.treeview_aptinstall.append_column(col_desc)
        self.treeview_aptinstall.connect('button-press-event', self.show_context_menu)
        
    def init_debstore(self):
        self.debsource_store = gtk.ListStore(gobject.TYPE_STRING,
                                             gobject.TYPE_PYOBJECT)
        self.treeview_debinstall.set_model(self.debsource_store)
        
        cell_desc = gtk.CellRendererText()
        cell_desc.set_property('xpad', 6)
        cell_desc.set_property('ypad', 6)
        col_desc = gtk.TreeViewColumn('Application', cell_desc, markup=0)
        #col_desc.set_max_width(1000)
        
        #self.treeview_debinstall.set_search_column(COLUMN_DESC)
        self.treeview_debinstall.append_column(col_desc)
        
    def fill_aptstore(self):
        self.aptsource_store.clear()
        for source in self.aptsources:
            self.aptsource_store.append([False,
                                         "<b>%s</b>" % source.get_name(),
                                         source])
        
        (path_x, path_y) = self.treeview_aptinstall.get_cursor()
        if len(self.aptsource_store) < 1 or path_x <0:
            self.button_aptedit.set_sensitive(False)
            self.button_aptremove.set_sensitive(False)
    
    def fill_debstore(self):
        self.debsource_store.clear()
        for source in self.debsources:
            self.debsource_store.append(["<b>%s</b> %s" % (source.get_name(), source.get_version()),
                                         source])
            
        (path_x, path_y) = self.treeview_debinstall.get_cursor()
        if len(self.debsource_store) < 1 or path_x <0:
            self.button_debedit.set_sensitive(False)
            self.button_debremove.set_sensitive(False)
    
    def refresh_install_count(self):
        self.install_count = len(self.install_dict)
        self.refresh_button_sensitive()
    
    def refresh_button_sensitive(self):
        if self.install_count == 0:
            self.button_aptinstall.set_sensitive(False)
        else:
            self.button_aptinstall.set_sensitive(True)
            
    def setBusy(self, flag):
        """ Show a watch cursor if the app is busy for more than 0.3 sec.
        Furthermore provide a loop to handle user interface events """
        if self.window_main.window is None:
            return
        if flag == True:
            self.window_main.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
        else:
            self.window_main.window.set_cursor(None)
        self.update()
    
    def update(self):
        while gtk.events_pending():
            gtk.main_iteration()
            
    def on_toggled(self, cell_toggle, path):
        self.aptsource_store[path][COLUMN_TOGGLE] = not self.aptsource_store[path][COLUMN_TOGGLE]
        
        if self.aptsource_store[path][COLUMN_TOGGLE] == True:
            self.install_dict[path] = self.aptsource_store
        else:
            if path in self.install_dict:
                del self.install_dict[path]
        self.refresh_install_count()
        return
    
    def on_treeview_aptinstall_row_activated(self, treeview, path, column):
        self.on_toggled(None, path)
    
    def on_treeview_debinstall_row_activated(self, treeview, path, column):
        pass
    
    def on_treeview_aptinstall_cursor_changed(self, treeview):
        selection = treeview.get_selection()
        (model, iter) = selection.get_selected()
        if iter:
            self.button_aptedit.set_sensitive(True)
            self.button_aptremove.set_sensitive(True)
        else:
            self.button_aptedit.set_sensitive(False)
            self.button_aptremove.set_sensitive(False)
        
    def on_treeview_debinstall_cursor_changed(self, treeview):
        selection = treeview.get_selection()
        (model, iter) = selection.get_selected()
        if iter:
            self.button_debinstall.set_sensitive(True)
            self.button_debedit.set_sensitive(True)
            self.button_debremove.set_sensitive(True)
        else:
            self.button_debinstall.set_sensitive(False)
            self.button_debedit.set_sensitive(False)
            self.button_debremove.set_sensitive(False)

    def show_context_menu(self, widget, event):
        if event.type == gtk.gdk.BUTTON_PRESS and event.button == 3:
            menu = gtk.Menu()
            item_select_none = gtk.MenuItem(_('전체 선택 취소(_U)'))
            item_select_none.connect('activate', self.select_none_install)
            menu.add(item_select_none)
            
            if self.install_count == 0:
                item_select_none.set_sensitive(False)
                
            item_select_all = gtk.MenuItem(_('전체 선택(_C)'))
            item_select_all.connect('activate', self.select_all_install)
            menu.add(item_select_all)
            
            menu.popup(None, None, None, 0, event.time)
            menu.show_all()
            return True        
    
    def select_none_install(self, widget):
        iter = self.aptsource_store.get_iter_first()
        self.install_dict.clear()
        while iter:
            self.aptsource_store.set_value(iter, COLUMN_TOGGLE, False)
            iter = self.aptsource_store.iter_next(iter)
        self.refresh_install_count()
        #self.treeview_update.queue_draw()
    
    def select_all_install(self, widget):
        iter = self.aptsource_store.get_iter_first()
        self.install_dict.clear()
        while iter:
            self.aptsource_store.set_value(iter, COLUMN_TOGGLE, True)
            path = self.aptsource_store.get_path(iter)
            self.install_dict[path] = self.aptsource_store
            iter = self.aptsource_store.iter_next(iter)
        self.refresh_install_count()
        #self.treeview_update.queue_draw()

    def on_button_aptinstall_clicked(self, widget):
        packages = []
        repositories = []
        keys = []
        for key, store in self.install_dict.items():
            iter = store.get_iter(key)
            source = store.get_value(iter, STORE_SOURCE)
            pkg = source.get_package()
            pkg = pkg.split()
            packages.extend(pkg)
            key = source.get_key()
            if key:
                keys.append(key)
            bin = source.get_bin_repo()
            if bin:
                repositories.append(bin)
            src = source.get_src_repo()
            if src:
                repositories.append(src)
                
        self.setBusy(True)
        apt = AptWorker()
        if len(repositories) != 0:
            apt.write_sourceslist(repositories)
        if len(keys) != 0:
            apt.add_key(keys)
        apt.perform_action(self.window_main, None, action=apt.UPDATE)
        if os.geteuid() == 0: apt.perform_action(self.window_main, None, packages, [], action=apt.INSTALL)
        self.setBusy(False)
        
    def on_button_debinstall_clicked(self, widget):
        selection = self.treeview_debinstall.get_selection()
        (store, iter) = selection.get_selected()
        if not iter:
            return
        self.setBusy(True)
        deb = DebWorker(self.datadir)
        source = store.get_value(iter, STORE_DEBSOURCE)
        pkg = source.get_package()
        url = source.get_url()
        ver = source.get_version()

        deb.perform_action(self.window_main, None, url)
        self.setBusy(False)
        
    def on_button_aptadd_clicked(self, widget):
        dialog = DialogAptSourceAdd(self.window_main, self.datadir)
        source = dialog.run()
        if source:
            self.insert_aptsource(source)
            self.get_aptsources()
            self.fill_aptstore()
        
    def on_button_aptedit_clicked(self, widget):
        selection = self.treeview_aptinstall.get_selection()
        (store, iter) = selection.get_selected()
        if not iter:
            return
        
        source_original = store.get_value(iter, STORE_SOURCE)
        dialog = DialogAptSourceEdit(self.window_main, source_original, self.datadir)
        source_changed = dialog.run()
        if source_changed:
            self.update_aptsource(source_changed, source_original.get_name())
            self.get_aptsources()
            self.fill_aptstore()
    
    def on_button_aptremove_clicked(self, widget):
        selection = self.treeview_aptinstall.get_selection()
        (store, iter) = selection.get_selected()
        if not iter:
            return
        
        source = store.get_value(iter, STORE_SOURCE)
        d = gtk.MessageDialog(flags=gtk.DIALOG_MODAL,
                                  type=gtk.MESSAGE_WARNING,
                                  buttons=gtk.BUTTONS_OK_CANCEL)
        d.set_title("")
        d.set_markup("<big><b>%s</b></big>\n\n%s" % (source.get_name(), "삭제하시겠습니까?"))
        d.realize()
        #d.window.set_functions(gtk.gdk.FUNC_MOVE)
        if d.run() == gtk.RESPONSE_OK:
            self.delete_aptsource(source)
            self.get_aptsources()
            self.fill_aptstore()
        d.destroy()
    
    def on_button_debadd_clicked(self, widget):
        dialog = DialogDebSourceAdd(self.window_main, self.datadir)
        source = dialog.run()
        if source:
            self.insert_debsource(source)
            self.get_debsources()
            self.fill_debstore()
    
    def on_button_debedit_clicked(self, widget):
        selection = self.treeview_debinstall.get_selection()
        (store, iter) = selection.get_selected()
        if not iter:
            return
        
        source_original = store.get_value(iter, STORE_DEBSOURCE)
        dialog = DialogDebSourceEdit(self.window_main, source_original, self.datadir)
        source_changed = dialog.run()
        if source_changed:
            self.update_debsource(source_changed, source_origianl.get_name())
            self.get_debsources()
            self.fill_debstore()
    
    def on_button_debremove_clicked(self, widget):
        selection = self.treeview_debinstall.get_selection()
        (store, iter) = selection.get_selected()
        if not iter:
            return
        
        source = store.get_value(iter, STORE_DEBSOURCE)
        d = gtk.MessageDialog(flags=gtk.DIALOG_MODAL,
                                  type=gtk.MESSAGE_WARNING,
                                  buttons=gtk.BUTTONS_OK_CANCEL)
        d.set_title("")
        d.set_markup("<big><b>%s</b></big>\n\n%s" % (source.get_name(), "삭제하시겠습니까?"))
        d.realize()
        #d.window.set_functions(gtk.gdk.FUNC_MOVE)
        if d.run() == gtk.RESPONSE_OK:
            self.delete_debsource(source)
            self.get_debsources()
            self.fill_debstore()
        d.destroy()
    
    
    def on_button_about_clicked(self, widget):
        self.dialog_about.set_version(VERSION)
        self.dialog_about.run()
        self.dialog_about.hide()
        
    def on_button_close_clicked(self, widget):
        gtk.main_quit()
        
    def on_window_main_delete_event(self, window, event):
        self.on_button_close_clicked(None)
        sys.exit()
        
def main():
    gtk.main()
    return 0

if __name__ == '__main__':
    VowelMainGtk("../")
    main()