#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vowel - Check.py

import os
import socket
from subprocess import Popen, PIPE

class Network:
    def test(self):
        """ Test network connectivity """
        try:
            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            s.connect(('www.google.com', 80))
            return True
        except socket.error:
            return False

class Architecture:
    def distroname(self):
        """ Get my disributor ID(ubuntu, kubuntu, ans so on...) """
        p1 = Popen(['lsb_release', '-i', '-s'], stdout=PIPE)
        p = p1.communicate()[0]
        
        try:
            mydistro = p.strip()
            #tokens = name.split('\t')    
        except AttributeError:
            print 'ERROR: Please install lsb_release'
            sys.exit()
        
        mydistro = mydistro.lower()
        return mydistro
    

    def codename(self):
        """ Get my code name of this distribution(hardy, gutsy, feisty, and so on...) """
        p1 = Popen(['lsb_release', '-c', '-s'], stdout=PIPE)
        p = p1.communicate()[0]
        
        try:
            mycode = p.strip()
            #tokens = name.split('\t')    
        except AttributeError:
            print 'ERROR: Please install lsb-release'
            sys.exit()
        
        mycode = mycode.lower()
        return mycode
    
    def machinename(self):
        """ Get my machine hardware name(x86, x86_64, powerpc, and so on...) """
        mymachine = os.uname()[-1]
        if mymachine == 'x86_64':
            return 'x86_64'
        elif mymachine in ('i386', 'i586', 'i686', 'k7'):
            return 'x86'
        elif mymachine in ('ppc', 'ppc64'):
            return 'powerpc'
        else:
            return mymachine
    
    def check_distro(self, mydistro, distro):
        """ Check my distributor id """
        distro = distro.split()
        if mydistro in distro: return True
        else: return False
    
    def check_code(self, mycode, code):
        """ Check my code name """
        code = code.split()
        if mycode in code: return True
        else: return False
    
    def check_machine(self, mymachine, machine):
        """ Check my machine name """
        machine = machine.split()
        if mymachine in machine: return True
        else: return False
 
class Package:
    # Initialize
    def __init__(self, package):
        self.__package = package
        self.__mypkgname, self.__mypkgversion = self.pkg_info()
    
    def pkg_info(self):
        """ Get package information(installed package's name and version) """
        p1 = os.popen('dpkg -l')
        p2 = Popen(['grep', self.__package], stdin=p1, stdout=PIPE)
        
        p = p2.communicate()[0]
        try:
            if len(p) == 0:
                return None, None
            else:
                tokens = p.strip().split()
                mypkgname = tokens[1].lower()
                mypkgversion = tokens[2].lower()
                return mypkgname, mypkgversion
        
        except AttributeError:
            print 'ERROR: Please install dpkg, grep'
            sys.exit()
    
    def isinstalled(self):
        """ Check if the package is installed """
        if self.__mypkgname is None or self.__mypkgname != self.__package: return False
        else: return True
    
    def islatest(self, version):
        """ Check if the package is latest """
        if(cmp(self.__mypkgversion, version) in (0, 1)): return True
        else: return False

class Dependency:
    def cmdoutput(self, command, verbose=None):
        from subprocess import STDOUT
        if verbose == 'noverbose':
            NULL = open('/dev/null', 'w')
            comnd = Popen(command, shell=True, stdout=PIPE, stderr=NULL)
            sts = os.waitpid(comnd.pid, 0)
            NULL.close()
        else:
            comnd = Popen(command + ' 2>&1', shell=True)# stdout=PIPE)
            #comnd = Popen(command, shell=True, stdout=PIPE, stderr=STDOUT)
            sts = os.waitpid(comnd.pid, 0)
            #os.system(command)
        #commandnew = comnd.communicate()[0]
        
    def checkpkg(self, pkglist):
        lines = []
        notinstalled = []
        p1 = Popen(['dpkg', '--get-selections'], stdout=PIPE)
        p = p1.communicate()[0]
        c = p.split('\n')
        for line in c:
            if line.find('\tinstall') != -1:#the relevant lines
                lines.append(line.split('\t')[0])
        for pkg in pkglist:
            try:
                if lines.index(pkg):
                    pass
            except ValueError:
                notinstalled.append(pkg)

        return notinstalled

    def pkginstall(self, pkglist):
        sep = ' '
        notinstalled = self.checkpkg(pkglist)
        if len(notinstalled) != 0:
            print 'Vowel: 다음 꾸러미가 설치되지 않았습니다'
            print '\n'.join(notinstalled)#for x in a: print x
            print '\nVowel: 꾸러미를 설치할 것입니다'
            packages = sep.join(notinstalled)
            self.cmdoutput('sudo apt-get --allow-unauthenticated --assume-yes install ' + packages)
        else:
            pass
    '''        
    def pkgupdate(self):
        self.cmdoutput('sudo apt-get update')
    '''        
    def checkntry(self, pkglist):
        self.pkginstall(pkglist)
        notinstalled = self.checkpkg(pkglist)#check again
        if len(notinstalled) != 0:
            error = '\nERROR: 다음 꾸러미는 설치할 수 없습니다', 
            '\n'.join(notinstalled)
            
            raise Exception(error)