#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vowel - DebWorker.py

import gtk
import gtk.glade

import os
import time
import threading
import subprocess
import urllib
from urllib import urlretrieve

from SimpleGladeApp import SimpleGladeApp


class DebWorker(SimpleGladeApp):
    def __init__(self, datadir):
        SimpleGladeApp.__init__(self, path=datadir+'/glade/main.glade', root=None, domain='vowel')
        self.window_download.realize()
        self.window_download.show()
        
        self.cancel = threading.Event()
        self._file = None
        self.canStart = True
        
    def download(self, url, lock):
        """ Download a deb package through given url """
        import urlparse
        self.label_download.set_text("꾸러미 정보를 얻는 중...")
        u = urlparse.urlparse(url)
        if os.path.splitext(u[2])[1] == '.deb':
            self._file = os.path.split(u[2])[1]
            
        u = urllib.splittype(url)
        host, selector = urllib.splithost(u[1])
        
        from httplib import HTTP
        request = HTTP(host)
        request.putrequest('GET', selector)
        request.putheader('Host', host)
        request.endheaders()
        
        errorCode, errorMessage, headers = request.getreply()
        if errorCode == 200 and errorMessage == 'OK':
            try:
                self._file = headers['Content-Disposition'].split('\'\'')[1]
            except KeyError, e:
                pass
        elif errorCode == 302 and errorMessage == 'Found':
            url = headers['Location']
            u = urlparse.urlparse(url)
            if os.path.splitext(u[2])[1] == '.deb':
                self._file = os.path.split(u[2])[1]
        
        try:
            self.label_download.set_text(self._file+"을 다운로드 중...")
            os.chdir('/var/cache/apt/archives/')
            urlretrieve(url, self._file, reporthook=self.hook)
            self.window_download.destroy()
            lock.release()
        except UserWarning, e:
            self.window_download.destroy()
            lock.release()
            self.canStart = False
        
    def hook(self, block_number, block_size, total_size):
        if self.cancel.isSet():
            raise UserWarning
        fraction = float(block_number * block_size) / total_size
        fraction = max(min(fraction, 1.0), 0.0)
        percent = '%2.f%%' % (fraction*100)
        self.progressbar_download.set_fraction(fraction)
        self.progressbar_download.set_text(percent)
    
    def run_gdebi(self, id, lock, to_add=None,to_rm=None):
        """ Run the gdebi-gtk program for installing a downloaded package """
        #apt_pkg.PkgSystemUnLock()
        #print "run_synaptic(%s,%s,%s)" % (id, lock, selections)
        cmd = []
        if os.getuid() != 0:
            cmd = ["/usr/bin/gksu",
                   "--desktop", "/usr/share/applications/gdebi.desktop",
                   "--"]
        cmd += ["/usr/bin/gdebi-gtk",
                to_add]
        '''
        if action == INSTALL:
            # install the stuff
            for item in to_add:
                f.write("%s\tinstall\n" % item)
                #print item.pkgname
            for item in to_rm:
                f.write("%s\tuninstall\n" % item)
            cmd.append("--set-selections-file")
            cmd.append("%s" % f.name)
            f.flush()
            subprocess.call(cmd)
            f.close()
        elif action == UPDATE:
            #print "Updating..."
            cmd.append("--update-at-startup")
            subprocess.call(cmd)
        else:
            print "run_synaptic() called with unknown action"
            sys.exit(1)'''
            
        subprocess.call(cmd)
        lock.release()
        
    def perform_action(self, window_main, cache, url):
        """ Perform action installing deb packages """
        window_main.set_sensitive(False)
        cwd = os.getcwd()
        lock = threading.Lock()
        lock.acquire()
        t = threading.Thread(target=self.download,args=(url,lock))
        t.start()
        while lock.locked():
            while gtk.events_pending():
                gtk.main_iteration()
            time.sleep(0.05)
        
        if self.canStart and self._file:
            lock.acquire()
            t = threading.Thread(target=self.run_gdebi,args=(window_main.window.xid,lock,self._file,[]))
            t.start()
            while lock.locked():
                while gtk.events_pending():
                    gtk.main_iteration()
                time.sleep(0.05)
        os.chdir(cwd)
        window_main.set_sensitive(True)
                
    def on_button_download_cancel_clicked(self, widget):
        self.cancel.set()
    
    def on_window_download_delete_event(self, window, event):
        self.on_button_download_cancel_clicked(None)